@@ -27,8 +27,7 @@ import com.google.common.collect.Maps;
 import io.netty.buffer.Unpooled;
 import org.junit.Test;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.fail;
+import static org.junit.Assert.*;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyInt;
 import static org.mockito.ArgumentMatchers.anyLong;
@@ -46,6 +45,7 @@ import org.apache.spark.network.client.RpcResponseCallback;
 import org.apache.spark.network.client.TransportClient;
 import org.apache.spark.network.shuffle.protocol.BlockTransferMessage;
 import org.apache.spark.network.shuffle.protocol.FetchShuffleBlocks;
+import org.apache.spark.network.shuffle.protocol.FetchShuffleBlockChunks;
 import org.apache.spark.network.shuffle.protocol.OpenBlocks;
 import org.apache.spark.network.shuffle.protocol.StreamHandle;
 import org.apache.spark.network.util.MapConfigProvider;
@@ -243,6 +243,55 @@ public class OneForOneBlockFetcherSuite {
     }
   }
 
+  @Test
+  public void testShuffleBlockChunksFetch() {
+    LinkedHashMap<String, ManagedBuffer> blocks = Maps.newLinkedHashMap();
+    blocks.put("shuffleChunk_0_0_0_0", new NioManagedBuffer(ByteBuffer.wrap(new byte[12])));
+    blocks.put("shuffleChunk_0_0_0_1", new NioManagedBuffer(ByteBuffer.wrap(new byte[23])));
+    blocks.put("shuffleChunk_0_0_0_2",
+      new NettyManagedBuffer(Unpooled.wrappedBuffer(new byte[23])));
+    String[] blockIds = blocks.keySet().toArray(new String[blocks.size()]);
+
+    BlockFetchingListener listener = fetchBlocks(blocks, blockIds,
+      new FetchShuffleBlockChunks("app-id", "exec-id", 0, 0, new int[] { 0 },
+        new int[][] {{ 0, 1, 2 }}), conf);
+    for (int i = 0; i < 3; i ++) {
+      verify(listener, times(1)).onBlockFetchSuccess("shuffleChunk_0_0_0_" + i,
+        blocks.get("shuffleChunk_0_0_0_" + i));
+    }
+  }
+
+  @Test
+  public void testShuffleBlockChunkFetchFailure() {
+    LinkedHashMap<String, ManagedBuffer> blocks = Maps.newLinkedHashMap();
+    blocks.put("shuffleChunk_0_0_0_0", new NioManagedBuffer(ByteBuffer.wrap(new byte[12])));
+    blocks.put("shuffleChunk_0_0_0_1", null);
+    blocks.put("shuffleChunk_0_0_0_2",
+      new NettyManagedBuffer(Unpooled.wrappedBuffer(new byte[23])));
+    String[] blockIds = blocks.keySet().toArray(new String[blocks.size()]);
+
+    BlockFetchingListener listener = fetchBlocks(blocks, blockIds,
+      new FetchShuffleBlockChunks("app-id", "exec-id", 0, 0, new int[]{0}, new int[][]{{0, 1, 2}}),
+        conf);
+    verify(listener, times(1)).onBlockFetchSuccess("shuffleChunk_0_0_0_0",
+      blocks.get("shuffleChunk_0_0_0_0"));
+    verify(listener, times(1)).onBlockFetchFailure(eq("shuffleChunk_0_0_0_1"), any());
+    verify(listener, times(1)).onBlockFetchSuccess("shuffleChunk_0_0_0_2",
+      blocks.get("shuffleChunk_0_0_0_2"));
+  }
+
+  @Test
+  public void testInvalidShuffleBlockIds() {
+    assertThrows(IllegalArgumentException.class, () -> fetchBlocks(new LinkedHashMap<>(),
+      new String[]{"shuffle_0_0"},
+      new FetchShuffleBlocks("app-id", "exec-id", 0, new long[] { 0 },
+        new int[][] {{ 0 }}, false), conf));
+    assertThrows(IllegalArgumentException.class, () -> fetchBlocks(new LinkedHashMap<>(),
+      new String[]{"shuffleChunk_0_0_0_0_0"},
+      new FetchShuffleBlockChunks("app-id", "exec-id", 0, 0, new int[] { 0 },
+        new int[][] {{ 0 }}), conf));
+  }
+
   /**
    * Begins a fetch on the given set of blocks by mocking out the server side of the RPC which
    * simply returns the given (BlockId, Block) pairs.
